#ifndef TXTYPES_H
#define TXTYPES_H

#include <limits.h>			// defines UBYTE_MAX, USHRT_MAX, ULONG_MAX

////////////////////////////////////////////////////////////////////////////////
//	standerd types
////////////////////////////////////////////////////////////////////////////////

#ifndef  BOOL
	#ifdef _WIN32
		typedef int BOOL ;						// Booleans
	#else
		typedef unsigned char BOOL ;				// Booleans
	#endif
#endif

#ifndef FALSE
	enum { FALSE=0, TRUE=1 } ;
	#define FALSE FALSE
	#define TRUE TRUE
#endif

#define GLUE(a, b)     a##b
#define PORT(x)        GLUE(PORT, x)
#define PIN(x)         GLUE(PIN, x)
#define DDR(x)         GLUE(DDR, x)

typedef unsigned char  BYTE ;	//  8 Bit unsigned
typedef unsigned short WORD ;	// 16 Bit unsigned
typedef unsigned long  DWORD ;	// 32 Bit unsigned
typedef unsigned long long  QWORD ;		// 64 Bit unsigned
typedef unsigned int   UINT;	// compiler dependend unsigned

typedef unsigned char  UCHAR ;	//  8 Bit unsigned , alias for FATFS
typedef unsigned long  ULONG ;	// 32 Bit unsigned , alias for FATFS

#define BYTE_MAX  UCHAR_MAX		// maximum for BYTE
#define WORD_MAX  USHRT_MAX		// maximum for WORD
#define DWORD_MAX ULONG_MAX		// maximum for DWORD

typedef unsigned int	BIT ;	// for Bitfields


enum { eOK = 0 } ;

////////////////////////////////////////////////////////////////////////////////
//	NULL-pointer
////////////////////////////////////////////////////////////////////////////////

#ifndef NULL
#define NULL 0L
#endif

////////////////////////////////////////////////////////////////////////////////
//	return the number of elements in an array
////////////////////////////////////////////////////////////////////////////////

#define elementsof(a) (sizeof(a)/sizeof(*(a)))


////////////////////////////////////////////////////////////////////////////////
//	conversion-routines
////////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus

inline void QWORDtoLittleEndian ( BYTE * p, QWORD ddw )
{
	p[0] = (BYTE) ( ddw >>  0 ) ;
	p[1] = (BYTE) ( ddw >>  8 ) ;
	p[2] = (BYTE) ( ddw >> 16 ) ;
	p[3] = (BYTE) ( ddw >> 24 ) ;
	p[4] = (BYTE) ( ddw >> 32 ) ;
	p[5] = (BYTE) ( ddw >> 40 ) ;
	p[6] = (BYTE) ( ddw >> 48 ) ;
	p[7] = (BYTE) ( ddw >> 56 ) ;

}

inline void DWORDtoLittleEndian ( BYTE * p, DWORD dw )
{
	p[0] = (BYTE) ( dw >>  0 ) ;
	p[1] = (BYTE) ( dw >>  8 ) ;
	p[2] = (BYTE) ( dw >> 16 ) ;
	p[3] = (BYTE) ( dw >> 24 ) ;
}

inline void WORDtoLittleEndian ( BYTE * p, WORD w )
{
	p[0] = (BYTE) ( w >> 0 ) ;
	p[1] = (BYTE) ( w >> 8 ) ;
}

inline QWORD LittleEndianToQWORD ( const BYTE * p )
{
	return (WORD)p[0] + ((WORD)p[1]<<8) + ((DWORD)p[2]<<16) + ((DWORD)p[3]<<24) + ((QWORD)p[4]<<32) + ((QWORD)p[5]<<40) + ((QWORD)p[6]<<48) + ((QWORD)p[7]<<56);
}

inline DWORD LittleEndianToDWORD ( const BYTE * p )
{
	return (WORD)p[0] + ((WORD)p[1]<<8) + ((DWORD)p[2]<<16) + ((DWORD)p[3]<<24);
}

inline WORD LittleEndianToWORD ( const BYTE * p )
{
	return (WORD)((WORD)p[0] + ((WORD)p[1]<<8)) ;
}

inline void QWORDtoBigEndian ( BYTE * p, QWORD ddw )
{
	p[7] = (BYTE) ( ddw >> 0 ) ;
	p[6] = (BYTE) ( ddw >> 8 ) ;
	p[5] = (BYTE) ( ddw >> 16 ) ;
	p[4] = (BYTE) ( ddw >> 24 ) ;
	p[3] = (BYTE) ( ddw >> 32 ) ;
	p[2] = (BYTE) ( ddw >> 40 ) ;
	p[1] = (BYTE) ( ddw >> 48 ) ;
	p[0] = (BYTE) ( ddw >> 56 ) ;
}

inline void DWORDtoBigEndian ( BYTE * p, DWORD dw )
{
	p[3] = (BYTE) ( dw >>  0 ) ;
	p[2] = (BYTE) ( dw >>  8 ) ;
	p[1] = (BYTE) ( dw >> 16 ) ;
	p[0] = (BYTE) ( dw >> 24 ) ;
}

inline void WORDtoBigEndian ( BYTE * p, WORD w )
{
	p[1] = (BYTE) ( w >> 0 ) ;
	p[0] = (BYTE) ( w >> 8 ) ;
}
inline QWORD BigEndianToQWORD ( const BYTE * p )
{
	return (WORD)p[7] + ((WORD)p[6]<<8) + ((DWORD)p[5]<<16) + ((DWORD)p[4]<<24) + ((QWORD)p[3]<<32) + ((QWORD)p[2]<<40) + ((QWORD)p[1]<<48) + ((QWORD)p[0]<<56);
}

inline DWORD BigEndianToDWORD ( const BYTE * p )
{
	return (WORD)p[3] + ((WORD)p[2]<<8) + ((DWORD)p[1]<<16) + ((DWORD)p[0]<<24);
}

inline WORD BigEndianToWORD ( const BYTE * p )
{
	return (WORD)((WORD)p[1] + ((WORD)p[0]<<8)) ;
}

#endif // __cplusplus

#endif